@@ -38,20 +38,21 @@ from The Open Group.
  *	Warning, there be dragons here....
  */
 
+#include <stdint.h>
 #include <X11/Xlib.h>
 #include <X11/Xproto.h>		/* to declare xEvent */
 #include <X11/XlibConf.h>	/* for configured options like XTHREADS */
 
-#ifdef WIN32
-#define _XFlush _XFlushIt
+/* The Xlib structs are full of implicit padding to properly align members.
+   We can't clean that up without breaking ABI, so tell clang not to bother
+   complaining about it. */
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wpadded"
 #endif
 
-/*
- * If your BytesReadable correctly detects broken connections, then
- * you should NOT define XCONN_CHECK_FREQ.
- */
-#ifndef XCONN_CHECK_FREQ
-#define XCONN_CHECK_FREQ 256
+#ifdef WIN32
+#define _XFlush _XFlushIt
 #endif
 
 struct _XGC
@@ -197,16 +198,127 @@ struct _XDisplay
 		XGenericEventCookie *	/* in */,
 		XGenericEventCookie *   /* out*/);
 	void *cookiejar;  /* cookie events returned but not claimed */
+#ifndef LONG64
+	unsigned long last_request_read_upper32bit;
+	unsigned long request_upper32bit;
+#endif
+
+	struct _XErrorThreadInfo *error_threads;
+
+	XIOErrorExitHandler exit_handler;
+	void *exit_handler_data;
+        Bool in_ifevent;
 };
 
 #define XAllocIDs(dpy,ids,n) (*(dpy)->idlist_alloc)(dpy,ids,n)
 
 /*
- * define the following if you want the Data macro to be a procedure instead
+ * access "last_request_read" and "request" with 64bit
+ * warning: the value argument of the SET-macros must not
+ * have any side-effects because it may get called twice.
  */
-#ifdef CRAY
-#define DataRoutineIsProcedure
-#endif /* CRAY */
+#ifndef LONG64
+/* accessors for 32-bit unsigned long */
+
+#define X_DPY_GET_REQUEST(dpy) \
+    ( \
+        ((uint64_t)(((struct _XDisplay*)dpy)->request)) \
+	+ (((uint64_t)(((struct _XDisplay*)dpy)->request_upper32bit)) << 32) \
+    )
+
+#define X_DPY_SET_REQUEST(dpy, value) \
+    ( \
+        (((struct _XDisplay*)dpy)->request = \
+            (value) & 0xFFFFFFFFUL), \
+        (((struct _XDisplay*)dpy)->request_upper32bit = \
+            ((uint64_t)(value)) >> 32), \
+	(void)0 /* don't use the result */ \
+    )
+
+#define X_DPY_GET_LAST_REQUEST_READ(dpy) \
+    ( \
+        ((uint64_t)(((struct _XDisplay*)dpy)->last_request_read)) \
+        + ( \
+            ((uint64_t)( \
+                ((struct _XDisplay*)dpy)->last_request_read_upper32bit \
+            )) << 32 \
+        ) \
+    )
+
+#define X_DPY_SET_LAST_REQUEST_READ(dpy, value) \
+    ( \
+        (((struct _XDisplay*)dpy)->last_request_read = \
+            (value) & 0xFFFFFFFFUL), \
+        (((struct _XDisplay*)dpy)->last_request_read_upper32bit = \
+            ((uint64_t)(value)) >> 32), \
+	(void)0 /* don't use the result */ \
+    )
+
+/*
+ * widen a 32-bit sequence number to a 64 sequence number.
+ * This macro makes the following assumptions:
+ * - ulseq refers to a sequence that has already been sent
+ * - ulseq means the most recent possible sequence number
+ *   with these lower 32 bits.
+ *
+ * The following optimization is used:
+ * The comparison result is taken a 0 or 1 to avoid a branch.
+ */
+#define X_DPY_WIDEN_UNSIGNED_LONG_SEQ(dpy, ulseq) \
+    ( \
+        ((uint64_t)ulseq) \
+        + \
+        (( \
+            ((uint64_t)(((struct _XDisplay*)dpy)->request_upper32bit)) \
+            - (uint64_t)( \
+                (ulseq) > (((struct _XDisplay*)dpy)->request) \
+	    ) \
+        ) << 32) \
+    )
+
+#define X_DPY_REQUEST_INCREMENT(dpy) \
+    ( \
+        ((struct _XDisplay*)dpy)->request++, \
+        ( \
+            (((struct _XDisplay*)dpy)->request == 0) ? ( \
+                ((struct _XDisplay*)dpy)->request_upper32bit++ \
+	    ) : 0 \
+        ), \
+	(void)0 /* don't use the result */ \
+    )
+
+
+#define X_DPY_REQUEST_DECREMENT(dpy) \
+    ( \
+	( \
+            (((struct _XDisplay*)dpy)->request == 0) ? (\
+                ((struct _XDisplay*)dpy)->request--, /* wrap */ \
+                ((struct _XDisplay*)dpy)->request_upper32bit-- \
+            ) : ( \
+                ((struct _XDisplay*)dpy)->request-- \
+            ) \
+	), \
+	(void)0 /* don't use the result */ \
+    )
+
+#else
+/* accessors for 64-bit unsigned long */
+#define X_DPY_GET_REQUEST(dpy) \
+    (((struct _XDisplay*)dpy)->request)
+#define X_DPY_SET_REQUEST(dpy, value) \
+    ((struct _XDisplay*)dpy)->request = (value)
+
+#define X_DPY_GET_LAST_REQUEST_READ(dpy) \
+    (((struct _XDisplay*)dpy)->last_request_read)
+#define X_DPY_SET_LAST_REQUEST_READ(dpy, value) \
+    ((struct _XDisplay*)dpy)->last_request_read = (value)
+
+#define X_DPY_WIDEN_UNSIGNED_LONG_SEQ(dpy, ulseq) ulseq
+
+#define X_DPY_REQUEST_INCREMENT(dpy) ((struct _XDisplay*)dpy)->request++
+#define X_DPY_REQUEST_DECREMENT(dpy) ((struct _XDisplay*)dpy)->request--
+#endif
+
 
 #ifndef _XEVENT_
 /*
@@ -338,15 +450,15 @@ extern LockInfoPtr _Xglobal_lock;
  */
 #if defined(MALLOC_0_RETURNS_NULL) || defined(__clang_analyzer__)
 
-# define Xmalloc(size) malloc(((size) == 0 ? 1 : (size)))
-# define Xrealloc(ptr, size) realloc((ptr), ((size) == 0 ? 1 : (size)))
-# define Xcalloc(nelem, elsize) calloc(((nelem) == 0 ? 1 : (nelem)), (elsize))
+# define Xmalloc(size) malloc((size_t)((size) == 0 ? 1 : (size)))
+# define Xrealloc(ptr, size) realloc((ptr), (size_t)((size) == 0 ? 1 : (size)))
+# define Xcalloc(nelem, elsize) calloc((size_t)((nelem) == 0 ? 1 : (nelem)), (size_t)(elsize))
 
 #else
 
-# define Xmalloc(size) malloc((size))
-# define Xrealloc(ptr, size) realloc((ptr), (size))
-# define Xcalloc(nelem, elsize) calloc((nelem), (elsize))
+# define Xmalloc(size) malloc((size_t)(size))
+# define Xrealloc(ptr, size) realloc((ptr), (size_t)(size))
+# define Xcalloc(nelem, elsize) calloc((size_t)(nelem), (size_t)(elsize))
 
 #endif
 
@@ -400,25 +512,8 @@ extern LockInfoPtr _Xglobal_lock;
  * X Protocol packetizing macros.
  */
 
-/*   Need to start requests on 64 bit word boundaries
- *   on a CRAY computer so add a NoOp (127) if needed.
- *   A character pointer on a CRAY computer will be non-zero
- *   after shifting right 61 bits of it is not pointing to
- *   a word boundary.
- */
-#ifdef WORD64
-#define WORD64ALIGN if ((long)dpy->bufptr >> 61) {\
-           dpy->last_req = dpy->bufptr;\
-           *(dpy->bufptr)   = X_NoOperation;\
-           *(dpy->bufptr+1) =  0;\
-           *(dpy->bufptr+2) =  0;\
-           *(dpy->bufptr+3) =  1;\
-             dpy->request++;\
-             dpy->bufptr += 4;\
-         }
-#else /* else does not require alignment on 64-bit boundaries */
+/* Leftover from CRAY support - was defined empty on all non-Cray systems */
 #define WORD64ALIGN
-#endif /* WORD64 */
 
 /**
  * Return a len-sized request buffer for the request type. This function may
@@ -436,14 +531,8 @@ extern void *_XGetRequest(Display *dpy, CARD8 type, size_t len);
 /* GetReqSized is the same as GetReq but allows the caller to specify the
  * size in bytes. 'sz' must be a multiple of 4! */
 
-#if !defined(UNIXCPP) || defined(ANSICPP)
 #define GetReqSized(name, sz, req) \
 	req = (x##name##Req *) _XGetRequest(dpy, X_##name, sz)
-#else
-#define GetReqSized(name, sz, req) \
-	req = (x/**/name/**/Req *) _XGetRequest(dpy, X_/**/name, sz)
-#endif
-
 
 /*
  * GetReq - Get the next available X request packet in the buffer and
@@ -454,25 +543,14 @@ extern void *_XGetRequest(Display *dpy, CARD8 type, size_t len);
  *
  */
 
-#if !defined(UNIXCPP) || defined(ANSICPP)
 #define GetReq(name, req) \
 	GetReqSized(name, SIZEOF(x##name##Req), req)
-#else  /* non-ANSI C uses empty comment instead of "##" for token concatenation */
-#define GetReq(name, req) \
-	GetReqSized(name, SIZEOF(x/**/name/**/Req), req)
-#endif
 
 /* GetReqExtra is the same as GetReq, but allocates "n" additional
    bytes after the request. "n" must be a multiple of 4!  */
 
-#if !defined(UNIXCPP) || defined(ANSICPP)
 #define GetReqExtra(name, n, req) \
         GetReqSized(name, SIZEOF(x##name##Req) + n, req)
-#else
-#define GetReqExtra(name, n, req) \
-        GetReqSized(name, SIZEOF(x/**/name/**/Req) + n, req)
-#endif
-
 
 /*
  * GetResReq is for those requests that have a resource ID
@@ -480,49 +558,35 @@ extern void *_XGetRequest(Display *dpy, CARD8 type, size_t len);
  * "rid" is the name of the resource.
  */
 
-#if !defined(UNIXCPP) || defined(ANSICPP)
 #define GetResReq(name, rid, req) \
 	req = (xResourceReq *) _XGetRequest(dpy, X_##name, SIZEOF(xResourceReq)); \
-	req->id = (rid)
-#else
-#define GetResReq(name, rid, req) \
-	req = (xResourceReq *) _XGetRequest(dpy, X_/**/name, SIZEOF(xResourceReq)); \
-	req->id = (rid)
-#endif
+	if (req) req->id = (rid)
 
 /*
  * GetEmptyReq is for those requests that have no arguments
  * at all.
  */
-#if !defined(UNIXCPP) || defined(ANSICPP)
+
 #define GetEmptyReq(name, req) \
 	req = (xReq *) _XGetRequest(dpy, X_##name, SIZEOF(xReq))
-#else
-#define GetEmptyReq(name, req) \
-	req = (xReq *) _XGetRequest(dpy, X_/**/name, SIZEOF(xReq))
-#endif
 
-#ifdef WORD64
-#define MakeBigReq(req,n) \
-    { \
-    char _BRdat[4]; \
-    unsigned long _BRlen = req->length - 1; \
-    req->length = 0; \
-    memcpy(_BRdat, ((char *)req) + (_BRlen << 2), 4); \
-    memmove(((char *)req) + 8, ((char *)req) + 4, _BRlen << 2); \
-    memcpy(((char *)req) + 4, _BRdat, 4); \
-    Data32(dpy, (long *)&_BRdat, 4); \
-    }
-#else
+/*
+ * MakeBigReq sets the CARD16 "req->length" to 0 and inserts a new CARD32
+ * length, after req->length, before the data in the request.  The new length
+ * includes the "n" extra 32-bit words.
+ *
+ * Do not use MakeBigReq if there is no data already in the request.
+ * req->length must already be >= 2.
+ */
 #ifdef LONG64
 #define MakeBigReq(req,n) \
     { \
     CARD64 _BRdat; \
-    CARD32 _BRlen = req->length - 1; \
+    CARD32 _BRlen = (CARD32) (req->length - 1); \
     req->length = 0; \
     _BRdat = ((CARD32 *)req)[_BRlen]; \
-    memmove(((char *)req) + 8, ((char *)req) + 4, _BRlen << 2); \
-    ((CARD32 *)req)[1] = _BRlen + n + 2; \
+    memmove(((char *)req) + 8, ((char *)req) + 4, (_BRlen - 1) << 2); \
+    ((CARD32 *)req)[1] = _BRlen + (CARD32) (n) + 2; \
     Data32(dpy, &_BRdat, 4); \
     }
 #else
@@ -532,13 +596,19 @@ extern void *_XGetRequest(Display *dpy, CARD8 type, size_t len);
     CARD32 _BRlen = req->length - 1; \
     req->length = 0; \
     _BRdat = ((CARD32 *)req)[_BRlen]; \
-    memmove(((char *)req) + 8, ((char *)req) + 4, _BRlen << 2); \
-    ((CARD32 *)req)[1] = _BRlen + n + 2; \
+    memmove(((char *)req) + 8, ((char *)req) + 4, (_BRlen - 1) << 2); \
+    ((CARD32 *)req)[1] = _BRlen + (CARD32) (n) + 2; \
     Data32(dpy, &_BRdat, 4); \
     }
 #endif
-#endif
 
+/*
+ * SetReqLen increases the count of 32-bit words in the request by "n",
+ * or by "badlen" if "n" is too large.
+ *
+ * Do not use SetReqLen if "req" does not already have data after the
+ * xReq header.  req->length must already be >= 2.
+ */
 #ifndef __clang_analyzer__
 #define SetReqLen(req,n,badlen) \
     if ((req->length + n) > (unsigned)65535) { \
@@ -546,10 +616,10 @@ extern void *_XGetRequest(Display *dpy, CARD8 type, size_t len);
 	    MakeBigReq(req,n) \
 	} else { \
 	    n = badlen; \
-	    req->length += n; \
+	    req->length = (CARD16) (req->length + n); \
 	} \
     } else \
-	req->length += n
+	req->length = (CARD16) (req->length + n)
 #else
 #define SetReqLen(req,n,badlen) \
     req->length += n
@@ -566,17 +636,17 @@ extern void _XFlushGCCache(Display *dpy, GC gc);
  * 32 bit word alignment.  Transmit if the buffer fills.
  *
  * "dpy" is a pointer to a Display.
- * "data" is a pinter to a data buffer.
+ * "data" is a pointer to a data buffer.
  * "len" is the length of the data buffer.
  */
 #ifndef DataRoutineIsProcedure
 #define Data(dpy, data, len) {\
 	if (dpy->bufptr + (len) <= dpy->bufmax) {\
-		memcpy(dpy->bufptr, data, (int)len);\
-		dpy->bufptr += ((len) + 3) & ~3;\
+		memcpy(dpy->bufptr, data, (size_t)(len));\
+		dpy->bufptr += ((size_t)((len) + 3) & (size_t)~3);\
 	} else\
-		_XSend(dpy, data, len);\
-	}
+		_XSend(dpy, (_Xconst char*)(data), (long)(len));\
+}
 #endif /* DataRoutineIsProcedure */
 
 
@@ -598,33 +668,28 @@ extern void _XFlushGCCache(Display *dpy, GC gc);
     if (dpy->bufptr + (n) > dpy->bufmax) \
         _XFlush (dpy); \
     ptr = (type) dpy->bufptr; \
-    memset(ptr, '\0', n); \
+    memset(ptr, '\0', (size_t)(n)); \
     dpy->bufptr += (n);
 
-#ifdef WORD64
-#define Data16(dpy, data, len) _XData16(dpy, (short *)data, len)
-#define Data32(dpy, data, len) _XData32(dpy, (long *)data, len)
-#else
-#define Data16(dpy, data, len) Data((dpy), (char *)(data), (len))
+#define Data16(dpy, data, len) Data((dpy), (_Xconst char *)(data), (len))
 #define _XRead16Pad(dpy, data, len) _XReadPad((dpy), (char *)(data), (len))
 #define _XRead16(dpy, data, len) _XRead((dpy), (char *)(data), (len))
 #ifdef LONG64
-#define Data32(dpy, data, len) _XData32(dpy, (long *)data, len)
+#define Data32(dpy, data, len) _XData32(dpy, (_Xconst long *)(data), (unsigned)(len))
 extern int _XData32(
 	     Display *dpy,
-	     register long *data,
+	     _Xconst long *data,
 	     unsigned len
 );
 extern void _XRead32(
 	     Display *dpy,
-	     register long *data,
+	     long *data,
 	     long len
 );
 #else
-#define Data32(dpy, data, len) Data((dpy), (char *)(data), (len))
+#define Data32(dpy, data, len) Data((dpy), (_Xconst char *)(data), (long)(len))
 #define _XRead32(dpy, data, len) _XRead((dpy), (char *)(data), (len))
 #endif
-#endif /* not WORD64 */
 
 #define PackData16(dpy,data,len) Data16 (dpy, data, len)
 #define PackData32(dpy,data,len) Data32 (dpy, data, len)
@@ -693,19 +758,10 @@ extern void _XRead32(
 }
 
 
-#ifdef MUSTCOPY
-
-/* for when 32-bit alignment is not good enough */
-#define OneDataCard32(dpy,dstaddr,srcvar) \
-  { dpy->bufptr -= 4; Data32 (dpy, (char *) &(srcvar), 4); }
-
-#else
-
 /* srcvar must be a variable for large architecture version */
 #define OneDataCard32(dpy,dstaddr,srcvar) \
   { *(CARD32 *)(dstaddr) = (srcvar); }
 
-#endif /* MUSTCOPY */
 
 typedef struct _XInternalAsync {
     struct _XInternalAsync *next;
@@ -728,6 +784,11 @@ typedef struct _XInternalAsync {
     XPointer data;
 } _XAsyncHandler;
 
+/*
+ * This struct is part of the ABI and is defined by value
+ * in user-code. This means that we cannot make
+ * the sequence-numbers 64bit.
+ */
 typedef struct _XAsyncEState {
     unsigned long min_sequence_number;
     unsigned long max_sequence_number;
@@ -871,7 +932,7 @@ extern int _XError(
 );
 extern int _XIOError(
     Display*	/* dpy */
-) _X_NORETURN;
+);
 extern int (*_XIOErrorFunction)(
     Display*	/* dpy */
 );
@@ -882,7 +943,14 @@ extern int (*_XErrorFunction)(
 extern void _XEatData(
     Display*		/* dpy */,
     unsigned long	/* n */
-);
+) _X_COLD;
+extern void _XEatDataWords(
+    Display*		/* dpy */,
+    unsigned long	/* n */
+) _X_COLD;
+#if defined(__SUNPRO_C) /* Studio compiler alternative to "cold" attribute */
+# pragma rarely_called(_XEatData, _XEatDataWords)
+#endif
 extern char *_XAllocScratch(
     Display*		/* dpy */,
     unsigned long	/* nbytes */
@@ -1308,10 +1376,10 @@ extern int _XF86LoadQueryLocaleFont(
 );
 
 extern void _XProcessWindowAttributes (
-    register Display *dpy,
+    Display *dpy,
     xChangeWindowAttributesReq *req,
-    register unsigned long valuemask,
-    register XSetWindowAttributes *attributes);
+    unsigned long valuemask,
+    XSetWindowAttributes *attributes);
 
 extern int _XDefaultError(
         Display *dpy,
@@ -1320,8 +1388,12 @@ extern int _XDefaultError(
 extern int _XDefaultIOError(
         Display *dpy);
 
+extern void _XDefaultIOErrorExit(
+    Display *dpy,
+    void *user_data);
+
 extern void _XSetClipRectangles (
-    register Display *dpy,
+    Display *dpy,
     GC gc,
     int clip_x_origin, int clip_y_origin,
     XRectangle *rectangles,
@@ -1329,13 +1401,13 @@ extern void _XSetClipRectangles (
     int ordering);
 
 Status _XGetWindowAttributes(
-    register Display *dpy,
+    Display *dpy,
     Window w,
     XWindowAttributes *attr);
 
 int _XPutBackEvent (
-    register Display *dpy,
-    register XEvent *event);
+    Display *dpy,
+    XEvent *event);
 
 extern Bool _XIsEventCookie(
         Display *dpy,
@@ -1364,6 +1436,10 @@ extern void xlocaledir(
     int buf_len
 );
 
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
+
 _XFUNCPROTOEND
 
 #endif /* _X11_XLIBINT_H_ */
